<?php
namespace App;
use App\Handler;

class Connection {
  private $_ws;
  private $host;
  private $loop;

  public function __construct(String $host) {
    $this->loop = \React\EventLoop\Factory::create();
    $this->host = $host;
  }

  public function connect() {
    $reactConnector = new \React\Socket\Connector($this->loop, ['timeout' => 20]);
    $connector = new \Ratchet\Client\Connector($this->loop, $reactConnector);

    $connector($this->host)->then(
      array($this, "onConnectSuccess"),
      array($this, "onConnectError")
    );

    $this->loop->run();

    // \Ratchet\Client\connect($this->host)->then(
    //   array($this, "onConnectSuccess"),
    //   array($this, "onConnectError")
    // );
  }

  public function onConnectSuccess(\Ratchet\Client\WebSocket $webSocket) {
    $this->_ws = $webSocket;
    $webSocket->on('message', function($msg) {
      $str = $msg->getPayload();
      // echo PHP_EOL . "RECV:" . PHP_EOL . $str . PHP_EOL;
      Handler::trigger("INBOUND", $str);
    });

    $webSocket->on('close', function($code = null, $reason = null) {
      echo PHP_EOL . "SOCKET CLOSED!" . PHP_EOL;
    });

    $this->testWithPromise();
  }

  public function onConnectError(\Exception $error) {
    echo PHP_EOL . "SOCKET ERROR!" . PHP_EOL;
    print_r($error);
    $this->loop->stop();
  }

  public function send(String $message) {
    $promise = new \React\Promise\Promise(function (callable $resolve, callable $reject) {
      $callback = function($str) use ($resolve, $reject) {
        $resolve($str);
      };
      // Register a callback that will be fired by line 25 (Handler::trigger)
      Handler::registerOnce("INBOUND", $callback);
    });

    // echo PHP_EOL . "SEND:" . PHP_EOL . $message . PHP_EOL;
    $this->_ws->send($message);

    // return \React\Promise\Timer\timeout($promise, 10, \React\EventLoop\Factory::create());
    return $promise;
  }


  public function testWithPromise() {
    echo PHP_EOL . "Test without await ...." . PHP_EOL;
    $promise = $this->send('Hello World!')->then(
      function($result) {
        echo PHP_EOL . $result . PHP_EOL;
      },
      function($error) {}
    )->always(function() {

      $this->testUsingAwait();

    });
  }

  public function testUsingAwait() {
    echo PHP_EOL . "Test using await ...." . PHP_EOL;
    $promise = $this->send('Hello World!');
    // $loop = \React\EventLoop\Factory::create();
    $result = \Clue\React\Block\await($promise, $this->loop);
    echo PHP_EOL . $result . PHP_EOL;
  }
}
